/******************************************************************************
*
* project name:    Scorch
* file name:       Extern##.c
* initial date:    27.12.2004
* author:          mhubi@freesurf.ch
* description:     all functions to do with saving datas
*
*******************************************************************************
*
* Scorch -- another tank-game
* Copyright (C) 2004-2005  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/

#include "main-h13.h"


//get the pointer to a file (from Travis Fischer's Excitebike 68k)
unsigned char *GetDataPtr(const char *Filename, short Offset) {
	
	//variables
	unsigned char *Location;
	SYM_ENTRY *SymPtr;
	
	//search the file
	if ((SymPtr = SymFindPtr(SYMSTR(Filename), 0)) == NULL) {
		//error-handling if not found
		return NULL;
	}
	
	//grab the address
	if ((Location = (unsigned char*) HeapDeref(SymPtr->handle)) == NULL) {
		//error-handling
		return NULL;
	}
	
	//return the pointer
	return (Location + 2 + Offset);
	
}


//archives a file (from Travis Fischer's Excitebike 68k)
void Archive(const char *Filename) {
	
	//variables
	SYM_ENTRY *SymPtr = NULL;
	HSym hsym = SymFind(SYMSTR(Filename));
	
	SymPtr = DerefSym (hsym);
	
	//look if there could be garbage collection
	if (EM_findEmptySlot(*GetDataPtr(Filename, -2)) == NULL) {
		return;
	}
	//archive it (if not already archive)
	if (!SymPtr->flags.bits.archived) {
		
		EM_moveSymToExtMem(SYMSTR(Filename), HS_NULL);
		
	}
		
}


//unarchives a file (from Travis Fischer's Excitebike 68k)
void Unarchive(const char *Filename) {
	
	//variables
	SYM_ENTRY *SymPtr = NULL;
	HSym hsym = SymFind(SYMSTR(Filename));
	
	SymPtr = DerefSym (hsym);
	
	//if it's archive unarchive it
	if (SymPtr->flags.bits.archived) {
		
		EM_moveSymFromExtMem(SYMSTR(Filename), HS_NULL);
		
	}
	
}


//base taken from www.technoplaza.net, c-programming, file i/o
void SaveGame(void) {
	
	//variables
	FILE *f = NULL;
	SAVEGAME save_game;
	unsigned int success = TRUE;
	unsigned int loop1;
	
	//put the variables into the corresponding structure-variable
	for (loop1 = 0; loop1 < 5; loop1++) {
		save_game.player[loop1] = player[loop1];
	}
	memcpy(&save_game.game, game, sizeof(GAME));
	for (loop1 = 0; loop1 < 240; loop1++) {
		save_game.earth[loop1] = earth[loop1];
	}
	memcpy(&save_game.cloud, cloud, sizeof(CLOUD));
	
	//unarchive it
	Unarchive(gamefile);
	
	//delete an old file
	unlink(gamefile);
	
	//open the file for writing
	if ((f = fopen(gamefile, "wb")) == NULL) {
		
		//error-handling
		success = FALSE;
		
	}
	else {
		
		//write structure to the file
		if (fwrite(&save_game, sizeof(SAVEGAME), 1, f) != 1) {
			
			//error-handling
			success = FALSE;
			
		}
	
		//append the file id tag
		fputc(0, f);
		fputs("save", f);
		fputc(0, f);
		fputc(OTH_TAG, f);
		
		//close the file
		fclose(f);
		
	}
	
	//archive it
	Archive(gamefile);
	
	//handling the next-program-steps (success or not)
	if (success == TRUE) {
		
		//show success
		Rectangle();
		GrayDrawStrExt(0, CALC(16,30), (char*)"SUCCESS", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
		GrayDrawStrExt2B(0, CALC(36,50), (char*)"Game saved", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
		Sprite16_OR_R(CALC(71,111), CALC(56,70), 16, smiley[0], DPLANE);
		
		/*for (loop1 = 0; loop1 <= 500; loop1++) {
				pokeIO(0x600005, 0b10111);
		}*/
		Waitkey();
		
	}
	else {
		
		//show error
		Rectangle();
		GrayDrawStrExt(0, CALC(16,30), (char*)"FAILED", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
		GrayDrawStrExt2B(0, CALC(36,50), (char*)"Couldn't save game", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
		Sprite16_OR_R(CALC(71,111), CALC(56,70), 16, smiley[1], DPLANE);
		/*for (loop1 = 0; loop1 <= 500; loop1++) {
				pokeIO(0x600005, 0b10111);
		}*/
		Waitkey();
		
		//LeadOut(FASTQUIT);
		
	}
	
	
}


//base taken from www.technoplaza.net, c-programming, file i/o
void LoadGame(void) {
	
	//variables
	FILE *f = NULL;
	SAVEGAME save_game;
	unsigned int success = TRUE;
	unsigned int loop1;
	
	//open file for reading
	if ((f = fopen(gamefile, "rb")) == NULL) {
		
		//error-handling
		success = FALSE;
		
	}
	else {
		
		//read data from the file and put it into the save_option_structure
		if (fread(&save_game, sizeof(SAVEGAME), 1, f) != 1) {
			
			//error-handling
			success = FALSE;
			
		}

		//close the file
		fclose(f);
		
	}
	
	//if success, put the values into the game-variables; handling next program-steps
	if (success == TRUE) {
		
		for (loop1 = 0; loop1 < 5; loop1++) {
			player[loop1] = save_game.player[loop1];
		}
		memcpy(game, &save_game.game, sizeof(GAME));
		for (loop1 = 0; loop1 < 240; loop1++) {
			earth[loop1] = save_game.earth[loop1];
		}
		memcpy(cloud, &save_game.cloud, sizeof(CLOUD));
		
		//show success
		Rectangle();
		GrayDrawStrExt(0, CALC(16,30), (char*)"SUCCESS", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
		GrayDrawStrExt2B(0, CALC(36,50), (char*)"Game loaded", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
		Sprite16_OR_R(CALC(71,111), CALC(56,70), 16, smiley[0], DPLANE);
		/*for (loop1 = 0; loop1 <= 500; loop1++) {
				pokeIO(0x600005, 0b10111);
		}*/
		Waitkey();
		
	}
	else {
		
		//show error
		Rectangle();
		GrayDrawStrExt(0, CALC(16,30), (char*)"FAILED", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
		GrayDrawStrExt2B(0, CALC(36,50), (char*)"Couldn't load game", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
		Sprite16_OR_R(CALC(71,111), CALC(56,70), 16, smiley[1], DPLANE);
		/*for (loop1 = 0; loop1 <= 500; loop1++) {
				pokeIO(0x600005, 0b10111);
		}*/
		Waitkey();
		
		LeadOut(FASTQUIT);
		
	}
	
}

/* Decompresses an image that has been compressed with vertical RLE
 *
 * Paramenters:
 *   src = array of compressed data (pointer)
 *   dest = will become uncompressed array (pointer)
 *   size = size of decompressed data array
 *  src and dest need to be allocated before calling
 *  size needs to be initialized before calling
 *  change BYTES_PER_ROW and NUM_ROWS if needed
 */
#define SENTINEL_VALUE 0x91
//#define BYTES_PER_ROW 30
//#define NUM_ROWS 128

void DecompRLEVert(unsigned char *src, unsigned char *dest, int size, int bytes_per_row, int num_rows) {
	int i = 0, j;
	int pos = 0;
	int dcmpNum = 0;
	
	while (dcmpNum < size)
	{
		//decompress a run
		if (src[i] == SENTINEL_VALUE)
		{
			i++;
			for(j = 0; j < src[i + 1]; j++)
			{
				dest[pos] = src[i];
				pos += bytes_per_row;
				if (pos >= bytes_per_row * num_rows)
				{
					pos -= bytes_per_row * num_rows;
					pos++;
				}
			}
			i++;
			dcmpNum += src[i];
		}
		else
		{
			dest[pos] = src[i];
			pos += bytes_per_row;
			if (pos >= bytes_per_row * num_rows)
			{
				pos -= bytes_per_row * num_rows;
				pos++;
			}
			dcmpNum++;
		}
		i++;
	}
}


//base taken from www.technoplaza.net, c-programming, file i/o
void SaveOptions(void) {
	
	//variables
	FILE *f = NULL;
	SAVEGAME save_opt;
	unsigned int success = TRUE;
	unsigned int loop1;
	
	//put the variables into the corresponding structure-variable
	//memcpy(&save_opt, game, sizeof(GAME));
	/*for (loop1 = 0; loop1 < 6; loop1++) {
		save_opt.game.set[loop1] = game->set[loop1];
	}*/
	for (loop1 = 0; loop1 < 5; loop1++) {
		save_opt.player[loop1] = player[loop1];
	}
	memcpy(&save_opt.game, game, sizeof(GAME));
	
	/*save_opt.land_factor = game->set[LAND];
	save_opt.wind_factor = game->set[WIND];
	save_opt.start_values = game->set[START];
	save_opt.special_events = game->set[SPECIAL];
	save_opt.walls = game->set[WALLS];
	save_opt.weather = game->set[WEATHER];*/
	
	//unarchive it
	Unarchive(optfile);
	
	//delete an old file
	unlink(optfile);
	
	//open the file for writing
	if ((f = fopen(optfile, "wb")) == NULL) {
		
		//error-handling
		success = FALSE;
		
	}
	else {
		
		//write structure to the file
		if (fwrite(&save_opt, sizeof(SAVEGAME), 1, f) != 1) {
			
			//error-handling
			success = FALSE;
			
		}
	
		//append the file id tag
		fputc(0, f);
		fputs("save", f);
		fputc(0, f);
		fputc(OTH_TAG, f);
		
		//close the file
		fclose(f);
		
	}
	
	//archive it
	Archive(optfile);
	
	//handling the next-program-steps (success or not)
	if (success == TRUE) {
		
		//show success
		/*Rectangle();
		GrayDrawStrExt(0, CALC(16,30), (char*)"SUCCESS", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
		GrayDrawStrExt2B(0, CALC(46,60), (char*)"Options saved", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
		
		Waitkey();*/
		
		return;
		
	}
	else {
		
		//show error
		Rectangle();
		GrayDrawStrExt(0, CALC(16,30), (char*)"FAILED", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
		GrayDrawStrExt2B(0, CALC(36,50), (char*)"Couldn't save options", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
		Sprite16_OR_R(CALC(71,111), CALC(56,70), 16, smiley[1], DPLANE);
		/*for (loop1 = 0; loop1 <= 500; loop1++) {
				pokeIO(0x600005, 0b10111);
		}*/
		Waitkey();
		
		//LeadOut(FASTQUIT);
		
	}
	
	
}


//base taken from www.technoplaza.net, c-programming, file i/o
void LoadOptions(BOOL mode) {
	
	//variables
	FILE *f = NULL;
	SAVEGAME save_opt;
	unsigned int success = TRUE;
	unsigned int loop1;
	
	//open file for reading
	if ((f = fopen(optfile, "rb")) == NULL) {
		
		//error-handling
		success = FALSE;
		
	}
	else {
		
		//read data from the file and put it into the save_option_structure
		if (fread(&save_opt, sizeof(SAVEGAME), 1, f) != 1) {
			
			//error-handling
			success = FALSE;
			
		}

		//close the file
		fclose(f);
		
	}
	
	//if success, put the values into the game-variables; handling next program-steps
	if (success == TRUE) {
		
		if (mode == FALSE) {
			for (loop1 = 0; loop1 < 6; loop1++) {
				game->set[loop1] = save_opt.game.set[loop1];
			}
		}
		else if (mode == TRUE) {
			for (loop1 = 0; loop1 < 5; loop1++) {
				player[loop1] = save_opt.player[loop1];
			}
			memcpy(game, &save_opt.game, sizeof(GAME));
		}
		
		/*game->set[LAND] = save_opt.land_factor;
		game->set[WIND] = save_opt.wind_factor;
		game->set[START] = save_opt.start_values;
		game->set[SPECIAL] = save_opt.special_events;
		game->set[WALLS] = save_opt.walls;
		game->set[WEATHER] = save_opt.weather;*/
			
		//show success
		Rectangle();
		GrayDrawStrExt(0, CALC(16,30), (char*)"SUCCESS", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
		GrayDrawStrExt2B(0, CALC(36,50), (char*)"Options loaded", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
		Sprite16_OR_R(CALC(71,111), CALC(56,70), 16, smiley[0], DPLANE);
		/*for (loop1 = 0; loop1 <= 500; loop1++) {
				pokeIO(0x600005, 0b10111);
		}*/
		Waitkey();
		
	}
	else {
		
		//show error
		Rectangle();
		GrayDrawStrExt(0, CALC(16,30), (char*)"FAILED", A_NORMAL | A_CENTERED | A_SHADOWED, F_8x10);
		GrayDrawStrExt2B(0, CALC(36,50), (char*)"Couldn't load options", A_NORMAL | A_CENTERED, F_6x8, DPLANE, DPLANE);
		Sprite16_OR_R(CALC(71,111), CALC(56,70), 16, smiley[1], DPLANE);
		/*for (loop1 = 0; loop1 <= 500; loop1++) {
				pokeIO(0x600005, 0b10111);
		}*/
		Waitkey();
		
		if (mode == TRUE) {
			LeadOut(FASTQUIT);
		}
		
	}
	
	
}

//checks whether a hiscore file already exists
//prototype from www.technoplaza.net und C-Programming/Graduate Review (Nibbler-game)
unsigned int OptionsRequest(void) {
	
	//variables
	unsigned int needed = FALSE;
	//tigcclib: stdio.h
	FILE *f = fopen(optfile, "rb");
	
	//if we cannot open the file, we must announce it to the previous routine
	if (f == NULL) {
		needed = TRUE;
	}
	else {
		
		//close the file
		//changed function from www.technoplaza.net
		fclose(f);
		
	}

	return needed;
	
}

//ti89 version: completely done